//****************************************************************************
//                    Computing the structurality index for the Rössler system
//
//                                        Les Monts du Roumois, April 20, 2023
//****************************************************************************
# include <cstdlib>
# include <iostream>
# include <fstream>
# include <iomanip>
# include <cmath>
# include <ctime>

using namespace std;

  const double a = 0.492;
  const double b = 2.0;
  const double c = 4.0;
  const double dt = 0.01;                                         // Time step

  const int m = 3;                                    // state space dimension 
// **************** Rössler system *******************************************
#define fx(x,y,z) -y -z
#define fy(x,y,z) x + a*y
#define fz(x,y,z) b + z*(x-c) 

void Runge(double *v);                             // Fourth-order Runge-Kutta

int main ( )
{
  int i, j = 0, k = 0, px, py, Npix = 0;
  const int Ntrans = 50000;
  const int Npt = 10000;                              // number of data points
  const int Npixel = int(sqrt(double(Npt/40.0)));          // Number of pixels
  const double xp = (c-sqrt(c*c-4.0*a*b))/(2.0);       // Inner singular point
  double *v, *sv, yp, zp, syp, Struc;
  double Xmin = +1.0E+5, Xmax = -1.0E+5, dX;
  bool gate[Npixel][Npixel];

  v = new double[m];
  sv = new double[m];
//****************************************************************************
  ofstream data_12 ("rossler.dat");

  cout << " x_p = " << xp << "\n \n";
  for (px = 0; px < Npixel; px++)
  {
    for (py = 0; py < Npixel; py++)
    {
      gate[px][py] = true;
    }
  }
  v[0]=0.3;                                              // Initial conditions
  v[1]=0.3;
  v[2]=0.1;
  do                                          // Bounding the Poincaré section
  {
    for (i = 0; i < m; i++)
    {
      sv[i] = v[i];
    }
    j++;
    Runge(v);
    if ((j > Ntrans) && (v[0] >= xp && sv[0] < xp))
    {
      k++;
      yp = v[1] -((v[0]-xp)/(v[0]-sv[0]))*(v[1]-sv[1]);
      Xmin = fmin(Xmin,yp);
      Xmax = fmax(Xmax,yp);
    }
  } while (k < Npt);
  Xmin -= 0.05*Xmin;
  Xmax += 0.05*Xmax;
  syp = yp;
  cout << Xmin << " < x_n < " << Xmax << "\n \n";
  dX = (Xmax - Xmin)/double(Npixel);
  k = 0;
  do                                     // Computing the structurality index
  {
    for (i = 0; i < m; i++)
    {
      sv[i] = v[i];
    }
    j++;
    Runge(v);
    if (v[0] >= xp && sv[0] < xp)
    {
      k++;
      yp = v[1] -((v[0] - xp)/(v[0] - sv[0]))*(v[1] - sv[1]);
      px = int(round((syp - Xmin)/dX));
      py = int(round((yp - Xmin)/dX));
      if (k > 1) 
      {
        data_12 << syp << " " << yp << " " << "\n";
        if (px >= 0 && px < Npixel && py >= 0 && py < Npixel && gate[px][py]) 
	{
	  gate[px][py] = false;
	  Npix++;
	}
      }
      syp = yp;
    }
  } while (k < Npt);
  Struc = 1.0 - double(Npix)/(double(Npixel*Npixel));
  data_12.close ( );
  cout << " Struc = " << Struc << "\n";
  system("xmgrace rossler.dat -param rosmap.par"); 
  return 0;
}
// ***************************************************************************
   void Runge(double *v)                           // Fourth-order Runge-Kutta
// ***************************************************************************
   {
      int j;
      double phi[m], savev[m], FF[m];

      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        savev[j]=v[j];
        phi[j]=FF[j];
        v[j]=savev[j]+0.5*dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        phi[j] = phi[j] + 2.0*FF[j];
        v[j]=savev[j] + 0.5*dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        phi[j] = phi[j] + 2.0*FF[j];
        v[j] = savev[j] + dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        v[j] = savev[j] + (phi[j] + FF[j])*dt/6.0;
      }
   }
// ***************************************************************************
